precision highp float;

varying vec2 textureCoordinate;

uniform vec4 uGradientColor1;
uniform vec4 uGradientColor2;
uniform vec2 uGradientProgress;
uniform float uGradientDegree;
uniform float uOpacity;

uniform float uAspectRatio;

void main() {
    vec2 uv = textureCoordinate;
    uv.x *= uAspectRatio;

    vec2 center = vec2(uAspectRatio * 0.5, 0.5);
    vec2 direction = normalize(vec2(cos(uGradientDegree), sin(uGradientDegree)));

    vec2 absDir = abs(direction);
    vec2 sv = center / (absDir + step(absDir, vec2(0.0)) * 0.01);
    float stride = min(sv.x, sv.y);

    vec2 total = direction * stride * 2.0;
    vec2 start = center + total * (-0.5 + uGradientProgress.x);
    vec2 end = center + total * (-0.5 + uGradientProgress.y);
    center = (start + end) * 0.5;

    float halfLen = distance(start, center);
    float shadowLen = dot(uv - center, direction);
    gl_FragColor = mix(uGradientColor1, uGradientColor2,
                           mix(smoothstep(0.0, 1.0, (shadowLen + halfLen) / (halfLen * 2.0 + step(halfLen, 0.0))), step(0.0, shadowLen),
                                   step(halfLen, 0.0)));
    gl_FragColor.a = uOpacity;
}
